@@ -1,13 +1,13 @@
 """
 Formula Builder Tools for RMCP.
-
 Natural language to R formula conversion and validation.
 """
 
 import re
-from typing import Any, Dict, List
+from typing import Any, List
 
-from ..r_integration import execute_r_script
+from ..core.schemas import table_schema
+from ..r_integration import execute_r_script_async
 from ..registries.tools import tool
 
 
@@ -39,18 +39,86 @@ from ..registries.tools import tool
         },
         "required": ["description"],
     },
+    output_schema={
+        "type": "object",
+        "properties": {
+            "formula": {
+                "type": ["string", "null"],
+                "description": "Generated R formula",
+            },
+            "matched_pattern": {
+                "type": ["string", "null"],
+                "description": "Pattern that was matched to generate the formula",
+            },
+            "alternatives": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Alternative formula specifications",
+            },
+            "validation": {
+                "type": ["object", "null"],
+                "description": "Validation results if data was provided",
+                "properties": {
+                    "is_valid": {"type": "boolean"},
+                    "missing_variables": {"type": "array", "items": {"type": "string"}},
+                    "existing_variables": {
+                        "type": "array",
+                        "items": {"type": "string"},
+                    },
+                    "available_variables": {
+                        "type": "array",
+                        "items": {"type": "string"},
+                    },
+                    "variable_types": {
+                        "type": "object",
+                        "additionalProperties": {"type": "string"},
+                    },
+                    "warnings": {"type": "array", "items": {"type": "string"}},
+                    "formula_parsed": {"type": "boolean"},
+                    "error": {"type": "string"},
+                },
+            },
+            "examples": {
+                "type": "array",
+                "items": {
+                    "type": "object",
+                    "properties": {
+                        "description": {"type": "string"},
+                        "formula": {"type": "string"},
+                        "use_case": {"type": "string"},
+                    },
+                },
+                "description": "Example formulas for the analysis type",
+            },
+            "interpretation": {
+                "type": ["string", "null"],
+                "description": "Human-readable interpretation of the formula",
+            },
+            "suggestions": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Suggestions for improving the formula",
+            },
+        },
+        "required": [
+            "formula",
+            "matched_pattern",
+            "alternatives",
+            "examples",
+            "interpretation",
+            "suggestions",
+        ],
+        "additionalProperties": False,
+    },
     description="Convert natural language descriptions to R formulas",
 )
-async def build_formula(context, params):
+async def build_formula(context, params) -> dict[str, Any]:
     """Convert natural language to R formula."""
-
     description = params["description"].lower()
     analysis_type = params.get("analysis_type", "regression")
     data = params.get("data")
     suggest_alternatives = params.get("suggest_alternatives", True)
-
     await context.info("Building formula from description", description=description)
-
     # Define pattern mappings for natural language to formula conversion
     patterns = [
         # Basic prediction patterns
@@ -79,18 +147,15 @@ async def build_formula(context, params):
         (r"(\w+)\s+from\s+(\w+)\s+and\s+(.*)", r"\1 ~ \2 + \3"),
         (r"(\w+)\s+using\s+(\w+)\s+and\s+(.*)", r"\1 ~ \2 + \3"),
     ]
-
     # Apply pattern matching
     formula = None
     matched_pattern = None
-
     for pattern, replacement in patterns:
         match = re.search(pattern, description)
         if match:
             formula = re.sub(pattern, replacement, description)
             matched_pattern = pattern
             break
-
     # Clean up formula
     if formula:
         # Replace common words with R operators
@@ -99,35 +164,28 @@ async def build_formula(context, params):
         formula = re.sub(r"\s+with\s+", " + ", formula)
         formula = re.sub(r"\s+including\s+", " + ", formula)
         formula = re.sub(r"\s+,\s+", " + ", formula)
-
         # Clean whitespace
         formula = re.sub(r"\s+", " ", formula).strip()
-
     # If no pattern matched, try to extract variables manually
     if not formula:
         words = description.split()
         potential_vars = [w for w in words if w.isalnum() and len(w) > 1]
-
         if len(potential_vars) >= 2:
             # Assume first is outcome, rest are predictors
             outcome = potential_vars[0]
             predictors = " + ".join(potential_vars[1:])
             formula = f"{outcome} ~ {predictors}"
             matched_pattern = "manual extraction"
-
     # Generate alternative formulas if requested
     alternatives = []
     if suggest_alternatives and formula:
         alternatives = _generate_formula_alternatives(formula, analysis_type)
-
     # Validate formula syntax if data is provided
     validation_result = None
     if formula and data:
         validation_result = await _validate_formula(context, formula, data)
-
     # Create examples based on analysis type
     examples = _get_formula_examples(analysis_type)
-
     result = {
         "formula": formula,
         "matched_pattern": matched_pattern,
@@ -137,7 +195,6 @@ async def build_formula(context, params):
         "interpretation": _interpret_formula(formula) if formula else None,
         "suggestions": _get_improvement_suggestions(description, formula),
     }
-
     await context.info("Formula built successfully", formula=formula)
     return result
 
@@ -145,17 +202,13 @@ async def build_formula(context, params):
 def _generate_formula_alternatives(base_formula: str, analysis_type: str) -> List[str]:
     """Generate alternative formula specifications."""
     alternatives = []
-
     if "~" not in base_formula:
         return alternatives
-
     outcome, predictors = base_formula.split("~", 1)
     outcome = outcome.strip()
     predictors = predictors.strip()
-
     # Basic alternatives
     alternatives.append(f"{outcome} ~ {predictors}")  # Original
-
     # Add interaction terms
     if "+" in predictors and "*" not in predictors:
         pred_list = [p.strip() for p in predictors.split("+")]
@@ -165,69 +218,55 @@ def _generate_formula_alternatives(base_formula: str, analysis_type: str) -> Lis
             alternatives.append(
                 f"{outcome} ~ ({predictors}) + {pred_list[0]} * {pred_list[1]}"
             )
-
     # Add polynomial terms
     if "+" in predictors:
         pred_list = [p.strip() for p in predictors.split("+")]
         for pred in pred_list[:2]:  # Only first two to avoid complexity
             alternatives.append(f"{outcome} ~ {predictors} + I({pred}^2)")
-
     # Add all variables option
     alternatives.append(f"{outcome} ~ .")
-
     # Add intercept removal option
     alternatives.append(f"{outcome} ~ {predictors} - 1")
-
     return list(set(alternatives))  # Remove duplicates
 
 
-async def _validate_formula(context, formula: str, data: Dict) -> Dict:
+async def _validate_formula(context, formula: str, data: dict) -> dict[str, Any]:
     """Validate formula against provided data."""
-
     r_script = f"""
     # Convert data to data frame
     data <- as.data.frame(args$data)
     formula_str <- "{formula}"
-    
     # Parse formula
     tryCatch({{
         parsed_formula <- as.formula(formula_str)
-        
         # Extract variable names
         vars_in_formula <- all.vars(parsed_formula)
         vars_in_data <- names(data)
-        
         # Check which variables exist
         missing_vars <- vars_in_formula[!vars_in_formula %in% vars_in_data]
         existing_vars <- vars_in_formula[vars_in_formula %in% vars_in_data]
-        
         # Get variable types for existing variables
         var_types <- sapply(data[existing_vars], class)
-        
         # Check for potential issues
         warnings <- c()
-        
         # Check for missing values
         missing_counts <- sapply(data[existing_vars], function(x) sum(is.na(x)))
         high_missing <- names(missing_counts[missing_counts > 0.1 * nrow(data)])
-        
         if (length(high_missing) > 0) {{
             warnings <- c(warnings, paste("High missing values in:", paste(high_missing, collapse=", ")))
         }}
-        
         # Check for character variables (might need factors)
         char_vars <- names(var_types[var_types == "character"])
         if (length(char_vars) > 0) {{
             warnings <- c(warnings, paste("Character variables (consider converting to factors):", paste(char_vars, collapse=", ")))
         }}
-        
         result <- list(
             is_valid = length(missing_vars) == 0,
             missing_variables = missing_vars,
             existing_variables = existing_vars,
             available_variables = vars_in_data,
-            variable_types = var_types,
-            warnings = warnings,
+            variable_types = as.list(setNames(var_types, names(var_types))),
+            warnings = if(length(warnings) == 0) character(0) else warnings,
             formula_parsed = TRUE
         )
     }}, error = function(e) {{
@@ -238,9 +277,8 @@ async def _validate_formula(context, formula: str, data: Dict) -> Dict:
         )
     }})
     """
-
     try:
-        validation = execute_r_script(r_script, {"data": data})
+        validation = await execute_r_script_async(r_script, {"data": data})
         return validation
     except Exception as e:
         return {"is_valid": False, "error": str(e), "formula_parsed": False}
@@ -250,13 +288,10 @@ def _interpret_formula(formula: str) -> str:
     """Provide interpretation of the formula."""
     if not formula or "~" not in formula:
         return "Invalid formula format"
-
     outcome, predictors = formula.split("~", 1)
     outcome = outcome.strip()
     predictors = predictors.strip()
-
     interpretation = f"This formula models '{outcome}' as the outcome variable"
-
     if predictors == ".":
         interpretation += " using all other variables in the dataset as predictors."
     elif "+" in predictors:
@@ -271,20 +306,17 @@ def _interpret_formula(formula: str) -> str:
         interpretation += f" including interaction effects between {predictors.replace('*', ' and ')}."
     else:
         interpretation += f" with '{predictors}' as the predictor variable."
-
     # Add notes about special terms
     if "I(" in formula:
         interpretation += " The formula includes polynomial or transformation terms."
     if "- 1" in formula:
         interpretation += " The intercept term is excluded from the model."
-
     return interpretation
 
 
 def _get_improvement_suggestions(description: str, formula: str) -> List[str]:
     """Generate suggestions for improving the formula."""
     suggestions = []
-
     if not formula:
         suggestions.append(
             "Try rephrasing with clearer variable relationships (e.g., 'predict Y from X')"
@@ -293,28 +325,23 @@ def _get_improvement_suggestions(description: str, formula: str) -> List[str]:
             "Specify the outcome variable and predictor variables explicitly"
         )
         return suggestions
-
     # Check for common improvements
     if "+" not in formula and "*" not in formula and "~" in formula:
         suggestions.append(
             "Consider adding interaction terms with * operator (e.g., 'x1 * x2')"
         )
-
     if "time" in description.lower() and "time" not in formula.lower():
         suggestions.append("For time series data, consider adding time trends or lags")
-
     if any(word in description.lower() for word in ["control", "adjust", "account"]):
         suggestions.append("Use '. +' to include all variables plus specific controls")
-
     if "squared" in description.lower() or "quadratic" in description.lower():
         suggestions.append(
             "Add polynomial terms with I(variable^2) for curved relationships"
         )
-
     return suggestions
 
 
-def _get_formula_examples(analysis_type: str) -> List[Dict[str, str]]:
+def _get_formula_examples(analysis_type: str) -> List[dict[str, str]]:
     """Get example formulas for different analysis types."""
     examples = {
         "regression": [
@@ -401,7 +428,6 @@ def _get_formula_examples(analysis_type: str) -> List[Dict[str, str]]:
             },
         ],
     }
-
     return examples.get(analysis_type, examples["general"])
 
 
@@ -411,7 +437,7 @@ def _get_formula_examples(analysis_type: str) -> List[Dict[str, str]]:
         "type": "object",
         "properties": {
             "formula": {"type": "string", "description": "R formula to validate"},
-            "data": {"type": "object", "description": "Data to validate against"},
+            "data": table_schema(),
             "analysis_type": {
                 "type": "string",
                 "enum": ["regression", "anova", "correlation"],
@@ -420,17 +446,68 @@ def _get_formula_examples(analysis_type: str) -> List[Dict[str, str]]:
         },
         "required": ["formula", "data"],
     },
+    output_schema={
+        "type": "object",
+        "properties": {
+            "is_valid": {
+                "type": "boolean",
+                "description": "Whether the formula is valid",
+            },
+            "missing_variables": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Variables in formula but not in data",
+            },
+            "existing_variables": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Variables in formula that exist in data",
+            },
+            "available_variables": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "All variables available in the dataset",
+            },
+            "variable_types": {
+                "type": "object",
+                "description": "Data types of variables in the formula",
+                "additionalProperties": {"type": "string"},
+            },
+            "warnings": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Validation warnings about data quality or types",
+            },
+            "formula_parsed": {
+                "type": "boolean",
+                "description": "Whether the formula syntax is valid",
+            },
+            "error": {
+                "type": "string",
+                "description": "Error message if validation failed",
+            },
+            "suggestions": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Suggestions for fixing or improving the formula",
+            },
+            "analysis_type": {
+                "type": "string",
+                "enum": ["regression", "anova", "correlation"],
+                "description": "Type of analysis the formula is intended for",
+            },
+        },
+        "required": ["is_valid", "formula_parsed", "suggestions", "analysis_type"],
+        "additionalProperties": False,
+    },
     description="Validate R formula syntax and check against data",
 )
-async def validate_formula(context, params):
+async def validate_formula(context, params) -> dict[str, Any]:
     """Validate R formula against data."""
-
     formula = params["formula"]
     data = params["data"]
     analysis_type = params.get("analysis_type", "regression")
-
     await context.info("Validating formula", formula=formula)
-
     # Basic syntax validation
     if "~" not in formula:
         return {
@@ -438,10 +515,8 @@ async def validate_formula(context, params):
             "error": "Formula must contain '~' separator",
             "suggestions": ["Use format: outcome ~ predictor1 + predictor2"],
         }
-
     # Detailed validation using R
     validation = await _validate_formula(context, formula, data)
-
     # Add suggestions based on validation results
     suggestions = []
     if not validation.get("is_valid", False):
@@ -453,10 +528,8 @@ async def validate_formula(context, params):
                 suggestions.append(
                     f"Available variables: {', '.join(validation['available_variables'][:10])}"
                 )
-
     if validation.get("warnings"):
         suggestions.extend(validation["warnings"])
-
     # Add analysis-specific suggestions
     if analysis_type == "anova" and validation.get("variable_types"):
         numeric_predictors = [
@@ -468,8 +541,6 @@ async def validate_formula(context, params):
             suggestions.append(
                 f"For ANOVA, consider converting numeric predictors to factors: {', '.join(numeric_predictors)}"
             )
-
     validation["suggestions"] = suggestions
     validation["analysis_type"] = analysis_type
-
     return validation
