@@ -1,13 +1,12 @@
 """
 Error Recovery and Helper Tools for RMCP.
-
 Intelligent error diagnosis, data validation, and recovery suggestions.
 """
 
 import re
-from typing import Any, Dict, List
+from typing import Any
 
-from ..r_integration import execute_r_script
+from ..r_integration import execute_r_script_async
 from ..registries.tools import tool
 
 
@@ -35,22 +34,71 @@ from ..registries.tools import tool
         },
         "required": ["error_message"],
     },
+    output_schema={
+        "type": "object",
+        "properties": {
+            "error_type": {
+                "type": "string",
+                "enum": [
+                    "missing_package",
+                    "missing_variable",
+                    "formula_syntax",
+                    "file_not_found",
+                    "data_type",
+                    "missing_values",
+                    "memory_size",
+                    "general",
+                ],
+                "description": "Categorized type of error",
+            },
+            "suggestions": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Actionable suggestions to fix the error",
+            },
+            "data_suggestions": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Data-specific suggestions based on analysis",
+            },
+            "next_steps": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Recommended next steps to resolve the issue",
+            },
+            "documentation_links": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Relevant documentation and help links",
+            },
+            "quick_fixes": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Quick fix commands or code snippets",
+            },
+        },
+        "required": [
+            "error_type",
+            "suggestions",
+            "data_suggestions",
+            "next_steps",
+            "documentation_links",
+            "quick_fixes",
+        ],
+        "additionalProperties": False,
+    },
     description="Analyze errors and suggest fixes with actionable solutions",
 )
-async def suggest_fix(context, params):
+async def suggest_fix(context, params) -> dict[str, Any]:
     """Analyze error and provide actionable solutions."""
-
     error_message = params["error_message"]
     tool_name = params.get("tool_name", "unknown")
     data = params.get("data")
     parameters = params.get("parameters")
-
     await context.info("Analyzing error", error=error_message, tool=tool_name)
-
     # Pattern-based error analysis
     suggestions = []
     error_type = "general"
-
     # R package errors
     if "there is no package called" in error_message:
         match = re.search(
@@ -64,16 +112,14 @@ async def suggest_fix(context, params):
                 "Run 'rmcp check-r-packages' to see all missing packages",
                 "Install all RMCP packages at once with the command in the documentation",
             ]
-
     # Data format errors
-    elif "object not found" in error_message.lower():
+    elif "object" in error_message.lower() and "not found" in error_message.lower():
         error_type = "missing_variable"
         suggestions = [
             "Check that all variable names in your formula exist in the data",
             "Use data_info tool to see available variables in your dataset",
             "Verify spelling of variable names (R is case-sensitive)",
         ]
-
     # Formula errors
     elif "invalid formula" in error_message.lower() or "~" in error_message:
         error_type = "formula_syntax"
@@ -82,7 +128,6 @@ async def suggest_fix(context, params):
             "Use build_formula tool for natural language formula creation",
             "Ensure variable names don't contain spaces or special characters",
         ]
-
     # File not found errors
     elif (
         "file not found" in error_message.lower()
@@ -94,7 +139,6 @@ async def suggest_fix(context, params):
             "Use absolute paths or ensure the file is in the current directory",
             "Verify file permissions (read access required)",
         ]
-
     # Data type errors
     elif (
         "invalid type" in error_message.lower()
@@ -106,7 +150,6 @@ async def suggest_fix(context, params):
             "Convert character variables to factors or numeric as appropriate",
             "Use data_info tool to check variable types in your dataset",
         ]
-
     # Missing value errors
     elif "missing values" in error_message.lower() or "na" in error_message.lower():
         error_type = "missing_values"
@@ -115,7 +158,6 @@ async def suggest_fix(context, params):
             "Use na.omit() in R or filter out missing values",
             "Check data quality with data_info tool",
         ]
-
     # Memory or size errors
     elif "memory" in error_message.lower() or "too large" in error_message.lower():
         error_type = "memory_size"
@@ -124,7 +166,6 @@ async def suggest_fix(context, params):
             "Use sampling to reduce dataset size for initial analysis",
             "Consider using more memory-efficient methods",
         ]
-
     # Generic suggestions if no specific pattern matched
     if not suggestions:
         suggestions = [
@@ -133,20 +174,17 @@ async def suggest_fix(context, params):
             "Try a simpler version of your analysis first",
             "Use validate_data tool to check your dataset for common issues",
         ]
-
     # Add tool-specific suggestions
     tool_specific_suggestions = _get_tool_specific_suggestions(tool_name, error_message)
     suggestions.extend(tool_specific_suggestions)
-
     # Data-specific suggestions if data provided
     data_suggestions = []
     if data:
         try:
             data_analysis = await _analyze_data_for_errors(context, data)
             data_suggestions = data_analysis.get("suggestions", [])
-        except:
+        except Exception:
             pass
-
     result = {
         "error_type": error_type,
         "suggestions": suggestions[:10],  # Limit to top 10
@@ -155,7 +193,6 @@ async def suggest_fix(context, params):
         "documentation_links": _get_documentation_links(tool_name, error_type),
         "quick_fixes": _get_quick_fixes(error_type),
     }
-
     await context.info(
         "Error analysis completed",
         error_type=error_type,
@@ -164,9 +201,8 @@ async def suggest_fix(context, params):
     return result
 
 
-def _get_tool_specific_suggestions(tool_name: str, error_message: str) -> List[str]:
+def _get_tool_specific_suggestions(tool_name: str, error_message: str) -> list[str]:
     """Get suggestions specific to the tool that failed."""
-
     tool_suggestions = {
         "linear_model": [
             "Ensure you have at least 2 data points for regression",
@@ -194,13 +230,11 @@ def _get_tool_specific_suggestions(tool_name: str, error_message: str) -> List[s
             "Ensure sufficient data points (>20 recommended)",
         ],
     }
-
     return tool_suggestions.get(tool_name, [])
 
 
-def _get_next_steps(error_type: str, tool_name: str) -> List[str]:
+def _get_next_steps(error_type: str, tool_name: str) -> list[str]:
     """Get recommended next steps based on error type."""
-
     next_steps_map = {
         "missing_package": [
             "Install missing R packages",
@@ -228,7 +262,6 @@ def _get_next_steps(error_type: str, tool_name: str) -> List[str]:
             "Clean data before analysis",
         ],
     }
-
     return next_steps_map.get(
         error_type,
         [
@@ -239,27 +272,23 @@ def _get_next_steps(error_type: str, tool_name: str) -> List[str]:
     )
 
 
-def _get_documentation_links(tool_name: str, error_type: str) -> List[str]:
+def _get_documentation_links(tool_name: str, error_type: str) -> list[str]:
     """Get relevant documentation links."""
-
     base_docs = [
         "Quick Start Guide: examples/quick_start_guide.md",
         "README: README.md",
     ]
-
     if tool_name in ["linear_model", "logistic_regression"]:
         base_docs.append("R regression documentation: ?lm, ?glm")
     elif tool_name in ["correlation_analysis"]:
         base_docs.append("R correlation documentation: ?cor")
     elif error_type == "missing_package":
         base_docs.append("R package installation: install.packages()")
-
     return base_docs
 
 
-def _get_quick_fixes(error_type: str) -> List[str]:
+def _get_quick_fixes(error_type: str) -> list[str]:
     """Get quick fix commands for common errors."""
-
     quick_fixes = {
         "missing_package": [
             'install.packages(c("jsonlite", "plm", "lmtest", "sandwich", "AER"))',
@@ -278,61 +307,49 @@ def _get_quick_fixes(error_type: str) -> List[str]:
             "Convert to factor: as.factor(variable)",
         ],
     }
-
     return quick_fixes.get(error_type, [])
 
 
-async def _analyze_data_for_errors(context, data: Dict) -> Dict:
+async def _analyze_data_for_errors(context, data: dict) -> dict[str, Any]:
     """Analyze data to identify potential issues."""
-
     r_script = """
     data <- as.data.frame(args$data)
-    
     # Basic data analysis
     n_rows <- nrow(data)
     n_cols <- ncol(data)
     col_names <- names(data)
-    
     # Check for potential issues
     issues <- c()
     suggestions <- c()
-    
     # Missing values
     missing_counts <- sapply(data, function(x) sum(is.na(x)))
     high_missing <- names(missing_counts[missing_counts > 0.1 * n_rows])
-    
     if (length(high_missing) > 0) {
         issues <- c(issues, "High missing values detected")
         suggestions <- c(suggestions, paste("High missing values in:", paste(high_missing, collapse=", ")))
     }
-    
     # Variable types
     var_types <- sapply(data, class)
     char_vars <- names(var_types[var_types == "character"])
-    
     if (length(char_vars) > 0) {
         suggestions <- c(suggestions, paste("Character variables may need conversion:", paste(char_vars, collapse=", ")))
     }
-    
     # Small sample size
     if (n_rows < 10) {
         issues <- c(issues, "Small sample size")
         suggestions <- c(suggestions, "Sample size is small - results may be unreliable")
     }
-    
     # Single column
     if (n_cols == 1) {
         issues <- c(issues, "Single variable")
         suggestions <- c(suggestions, "Only one variable - cannot perform relationship analysis")
     }
-    
     # Constant variables
     constant_vars <- names(data)[sapply(data, function(x) length(unique(x[!is.na(x)])) <= 1)]
     if (length(constant_vars) > 0) {
         issues <- c(issues, "Constant variables detected")
         suggestions <- c(suggestions, paste("Constant variables (no variation):", paste(constant_vars, collapse=", ")))
     }
-    
     result <- list(
         issues = issues,
         suggestions = suggestions,
@@ -344,9 +361,8 @@ async def _analyze_data_for_errors(context, data: Dict) -> Dict:
         )
     )
     """
-
     try:
-        analysis = execute_r_script(r_script, {"data": data})
+        analysis = await execute_r_script_async(r_script, {"data": data})
         return analysis
     except Exception:
         return {"issues": [], "suggestions": []}
@@ -377,29 +393,106 @@ async def _analyze_data_for_errors(context, data: Dict) -> Dict:
         },
         "required": ["data"],
     },
+    output_schema={
+        "type": "object",
+        "properties": {
+            "is_valid": {
+                "type": "boolean",
+                "description": "Whether the data passes validation",
+            },
+            "warnings": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Validation warnings that don't prevent analysis",
+            },
+            "errors": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Critical errors that prevent analysis",
+            },
+            "suggestions": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Suggestions for improving data quality",
+            },
+            "data_quality": {
+                "type": "object",
+                "properties": {
+                    "dimensions": {
+                        "type": "object",
+                        "properties": {
+                            "rows": {"type": "integer", "minimum": 0},
+                            "columns": {"type": "integer", "minimum": 0},
+                        },
+                    },
+                    "variable_types": {
+                        "type": "object",
+                        "properties": {
+                            "numeric": {"type": "integer", "minimum": 0},
+                            "character": {"type": "integer", "minimum": 0},
+                            "factor": {"type": "integer", "minimum": 0},
+                            "logical": {"type": "integer", "minimum": 0},
+                        },
+                    },
+                    "missing_values": {
+                        "type": "object",
+                        "properties": {
+                            "total_missing_cells": {"type": "integer", "minimum": 0},
+                            "variables_with_missing": {"type": "integer", "minimum": 0},
+                            "max_missing_percentage": {
+                                "type": "number",
+                                "minimum": 0,
+                                "maximum": 100,
+                            },
+                        },
+                    },
+                    "data_issues": {
+                        "type": "object",
+                        "properties": {
+                            "constant_variables": {"type": "integer", "minimum": 0},
+                            "high_outlier_variables": {"type": "integer", "minimum": 0},
+                            "duplicate_rows": {
+                                "type": ["integer", "null"],
+                                "minimum": 0,
+                            },
+                        },
+                    },
+                },
+                "description": "Detailed data quality assessment",
+            },
+            "recommendations": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Analysis-specific recommendations",
+            },
+        },
+        "required": [
+            "is_valid",
+            "warnings",
+            "errors",
+            "suggestions",
+            "data_quality",
+            "recommendations",
+        ],
+        "additionalProperties": False,
+    },
     description="Validate data quality and identify potential issues before analysis",
 )
-async def validate_data(context, params):
+async def validate_data(context, params) -> dict[str, Any]:
     """Validate data for analysis and identify potential issues."""
-
     data = params["data"]
     analysis_type = params.get("analysis_type", "general")
     strict = params.get("strict", False)
-
     await context.info("Validating data", analysis_type=analysis_type)
-
     r_script = f"""
     library(dplyr)
-    
     data <- as.data.frame(args$data)
     analysis_type <- "{analysis_type}"
     strict_mode <- {"TRUE" if strict else "FALSE"}
-    
     # Basic validation
     n_rows <- nrow(data)
     n_cols <- ncol(data)
     col_names <- names(data)
-    
     # Initialize results
     validation_results <- list(
         is_valid = TRUE,
@@ -408,88 +501,71 @@ async def validate_data(context, params):
         suggestions = c(),
         data_quality = list()
     )
-    
     # Check basic requirements
     if (n_rows == 0) {{
         validation_results$is_valid <- FALSE
         validation_results$errors <- c(validation_results$errors, "Dataset is empty (no rows)")
     }}
-    
     if (n_cols == 0) {{
         validation_results$is_valid <- FALSE
         validation_results$errors <- c(validation_results$errors, "Dataset has no columns")
     }}
-    
     # Missing value analysis
     missing_counts <- sapply(data, function(x) sum(is.na(x)))
     missing_percentages <- missing_counts / n_rows * 100
     high_missing <- names(missing_percentages[missing_percentages > 50])
-    
     if (length(high_missing) > 0) {{
-        validation_results$warnings <- c(validation_results$warnings, 
+        validation_results$warnings <- c(validation_results$warnings,
                                         paste("High missing values (>50%) in:", paste(high_missing, collapse=", ")))
     }}
-    
     # Variable type analysis
     var_types <- sapply(data, class)
     numeric_vars <- names(var_types[var_types %in% c("numeric", "integer")])
     character_vars <- names(var_types[var_types == "character"])
     factor_vars <- names(var_types[var_types == "factor"])
     logical_vars <- names(var_types[var_types == "logical"])
-    
     # Analysis-specific validation
     if (analysis_type == "regression") {{
         if (n_cols < 2) {{
             validation_results$errors <- c(validation_results$errors, "Regression requires at least 2 variables (outcome + predictor)")
         }}
-        
         if (n_rows < 10) {{
             validation_results$warnings <- c(validation_results$warnings, "Small sample size for regression (n < 10)")
         }}
-        
         if (length(numeric_vars) == 0) {{
             validation_results$warnings <- c(validation_results$warnings, "No numeric variables found - may need conversion")
         }}
     }}
-    
     if (analysis_type == "correlation") {{
         if (length(numeric_vars) < 2) {{
             validation_results$errors <- c(validation_results$errors, "Correlation requires at least 2 numeric variables")
         }}
-        
         if (n_rows < 3) {{
             validation_results$errors <- c(validation_results$errors, "Correlation requires at least 3 observations")
         }}
     }}
-    
     if (analysis_type == "timeseries") {{
         if (n_rows < 10) {{
             validation_results$warnings <- c(validation_results$warnings, "Small sample size for time series (n < 10)")
         }}
-        
         if (length(numeric_vars) == 0) {{
             validation_results$errors <- c(validation_results$errors, "Time series analysis requires numeric variables")
         }}
     }}
-    
     if (analysis_type == "classification") {{
         # Look for binary/categorical variables
         binary_vars <- names(data)[sapply(data, function(x) length(unique(x[!is.na(x)])) == 2)]
-        
         if (length(binary_vars) == 0 && length(factor_vars) == 0) {{
             validation_results$warnings <- c(validation_results$warnings, "No obvious outcome variable for classification found")
         }}
     }}
-    
     # Data quality checks
-    
     # Constant variables
     constant_vars <- names(data)[sapply(data, function(x) length(unique(x[!is.na(x)])) <= 1)]
     if (length(constant_vars) > 0) {{
-        validation_results$warnings <- c(validation_results$warnings, 
+        validation_results$warnings <- c(validation_results$warnings,
                                         paste("Constant variables (no variation):", paste(constant_vars, collapse=", ")))
     }}
-    
     # Outliers (for numeric variables)
     outlier_info <- list()
     for (var in numeric_vars) {{
@@ -499,62 +575,50 @@ async def validate_data(context, params):
             IQR <- Q3 - Q1
             outliers <- sum(data[[var]] < (Q1 - 1.5 * IQR) | data[[var]] > (Q3 + 1.5 * IQR), na.rm = TRUE)
             outlier_percentage <- outliers / sum(!is.na(data[[var]])) * 100
-            
             if (outlier_percentage > 10) {{
                 outlier_info[[var]] <- outlier_percentage
             }}
         }}
     }}
-    
     if (length(outlier_info) > 0) {{
-        validation_results$warnings <- c(validation_results$warnings, 
+        validation_results$warnings <- c(validation_results$warnings,
                                         paste("High outlier percentage in:", paste(names(outlier_info), collapse=", ")))
     }}
-    
     # Strict mode additional checks
     if (strict_mode) {{
         # Check for duplicate rows
         duplicate_rows <- sum(duplicated(data))
         if (duplicate_rows > 0) {{
-            validation_results$warnings <- c(validation_results$warnings, 
+            validation_results$warnings <- c(validation_results$warnings,
                                             paste("Duplicate rows found:", duplicate_rows))
         }}
-        
         # Check variable name issues
         problematic_names <- col_names[grepl("[^a-zA-Z0-9_.]", col_names)]
         if (length(problematic_names) > 0) {{
-            validation_results$warnings <- c(validation_results$warnings, 
+            validation_results$warnings <- c(validation_results$warnings,
                                             paste("Variable names with special characters:", paste(problematic_names, collapse=", ")))
         }}
-        
         # Check for very wide data
         if (n_cols > n_rows && n_rows < 100) {{
-            validation_results$warnings <- c(validation_results$warnings, 
+            validation_results$warnings <- c(validation_results$warnings,
                                             "More variables than observations - may be problematic")
         }}
     }}
-    
     # Generate suggestions
     suggestions <- c()
-    
     if (length(character_vars) > 0) {{
         suggestions <- c(suggestions, paste("Consider converting character variables to factors:", paste(character_vars[1:min(3, length(character_vars))], collapse=", ")))
     }}
-    
     if (any(missing_percentages > 10)) {{
         suggestions <- c(suggestions, "Consider handling missing values before analysis")
     }}
-    
     if (length(constant_vars) > 0) {{
         suggestions <- c(suggestions, "Remove constant variables as they don't contribute to analysis")
     }}
-    
     if (n_rows < 30) {{
         suggestions <- c(suggestions, "Small sample size - interpret results cautiously")
     }}
-    
     validation_results$suggestions <- suggestions
-    
     # Data quality summary
     validation_results$data_quality <- list(
         dimensions = list(rows = n_rows, columns = n_cols),
@@ -575,17 +639,17 @@ async def validate_data(context, params):
             duplicate_rows = if(strict_mode) duplicate_rows else NA
         )
     )
-    
+    # Ensure arrays are properly formatted for JSON
+    validation_results$warnings <- if(length(validation_results$warnings) == 0) character(0) else validation_results$warnings
+    validation_results$errors <- if(length(validation_results$errors) == 0) character(0) else validation_results$errors
+    validation_results$suggestions <- if(length(validation_results$suggestions) == 0) character(0) else validation_results$suggestions
     result <- validation_results
     """
-
     try:
-        result = execute_r_script(r_script, {"data": data})
-
+        result = await execute_r_script_async(r_script, {"data": data})
         # Add analysis-specific recommendations
         recommendations = _get_analysis_recommendations(analysis_type, result)
         result["recommendations"] = recommendations
-
         await context.info(
             "Data validation completed",
             is_valid=result["is_valid"],
@@ -593,7 +657,6 @@ async def validate_data(context, params):
             errors_count=len(result["errors"]),
         )
         return result
-
     except Exception as e:
         await context.error("Data validation failed", error=str(e))
         return {
@@ -607,50 +670,41 @@ async def validate_data(context, params):
 
 
 def _get_analysis_recommendations(
-    analysis_type: str, validation_result: Dict
-) -> List[str]:
+    analysis_type: str, validation_result: dict
+) -> list[str]:
     """Get analysis-specific recommendations based on validation results."""
-
     recommendations = []
     data_quality = validation_result.get("data_quality", {})
-
     if analysis_type == "regression":
         if data_quality.get("dimensions", {}).get("rows", 0) < 30:
             recommendations.append(
                 "For reliable regression results, consider collecting more data (n >= 30)"
             )
-
         if data_quality.get("variable_types", {}).get("numeric", 0) < 2:
             recommendations.append(
                 "Ensure you have numeric variables for regression analysis"
             )
-
     elif analysis_type == "correlation":
         if data_quality.get("variable_types", {}).get("numeric", 0) < 2:
             recommendations.append(
                 "Correlation analysis requires at least 2 numeric variables"
             )
-
         if data_quality.get("missing_values", {}).get("max_missing_percentage", 0) > 20:
             recommendations.append(
                 "High missing values may affect correlation estimates"
             )
-
     elif analysis_type == "timeseries":
         if data_quality.get("dimensions", {}).get("rows", 0) < 20:
             recommendations.append(
                 "Time series analysis works better with more observations (n >= 20)"
             )
-
     # General recommendations
     if data_quality.get("data_issues", {}).get("constant_variables", 0) > 0:
         recommendations.append("Remove constant variables before analysis")
-
     if data_quality.get("missing_values", {}).get("max_missing_percentage", 0) > 30:
         recommendations.append(
             "Consider imputation or removal of variables with high missing values"
         )
-
     return recommendations
 
 
@@ -678,24 +732,120 @@ def _get_analysis_recommendations(
         },
         "required": ["dataset_name"],
     },
+    output_schema={
+        "type": "object",
+        "properties": {
+            "data": {
+                "type": "object",
+                "description": "Example dataset in column-wise format",
+                "additionalProperties": {
+                    "type": "array",
+                    "items": {"type": ["string", "number", "boolean", "null"]},
+                },
+            },
+            "metadata": {
+                "type": "object",
+                "properties": {
+                    "name": {
+                        "type": "string",
+                        "enum": [
+                            "sales",
+                            "economics",
+                            "customers",
+                            "timeseries",
+                            "survey",
+                        ],
+                        "description": "Dataset name",
+                    },
+                    "description": {
+                        "type": "string",
+                        "description": "Dataset description and purpose",
+                    },
+                    "size": {
+                        "type": "string",
+                        "enum": ["small", "medium", "large"],
+                        "description": "Size category of the dataset",
+                    },
+                    "rows": {
+                        "type": "integer",
+                        "minimum": 1,
+                        "description": "Number of rows in the dataset",
+                    },
+                    "columns": {
+                        "type": "integer",
+                        "minimum": 1,
+                        "description": "Number of columns in the dataset",
+                    },
+                    "has_noise": {
+                        "type": "boolean",
+                        "description": "Whether noise/missing values were added",
+                    },
+                },
+                "description": "Dataset metadata and information",
+            },
+            "statistics": {
+                "type": "object",
+                "description": "Basic statistics for numeric variables",
+                "additionalProperties": {
+                    "type": "object",
+                    "properties": {
+                        "mean": {"type": "number"},
+                        "sd": {"type": "number", "minimum": 0},
+                        "min": {"type": "number"},
+                        "max": {"type": "number"},
+                        "missing": {"type": "integer", "minimum": 0},
+                    },
+                },
+            },
+            "suggested_analyses": {
+                "type": "array",
+                "items": {"type": "string"},
+                "description": "Suggested analyses appropriate for this dataset",
+            },
+            "variable_info": {
+                "type": "object",
+                "properties": {
+                    "numeric_variables": {
+                        "type": "array",
+                        "items": {"type": "string"},
+                        "description": "Names of numeric variables",
+                    },
+                    "categorical_variables": {
+                        "type": "array",
+                        "items": {"type": "string"},
+                        "description": "Names of categorical variables",
+                    },
+                    "variable_types": {
+                        "type": "object",
+                        "description": "Data type for each variable",
+                        "additionalProperties": {"type": "string"},
+                    },
+                },
+                "description": "Information about variables in the dataset",
+            },
+        },
+        "required": [
+            "data",
+            "metadata",
+            "statistics",
+            "suggested_analyses",
+            "variable_info",
+        ],
+        "additionalProperties": False,
+    },
     description="Load example datasets for testing and learning",
 )
-async def load_example(context, params):
+async def load_example(context, params) -> dict[str, Any]:
     """Load example datasets for analysis and testing."""
-
     dataset_name = params["dataset_name"]
     size = params.get("size", "small")
     add_noise = params.get("add_noise", False)
-
     await context.info("Loading example dataset", name=dataset_name, size=size)
-
     r_script = f"""
     library(dplyr)
-    
     dataset_name <- "{dataset_name}"
     size_param <- "{size}"
     add_noise <- {"TRUE" if add_noise else "FALSE"}
-    
     # Set size parameters
     size_map <- list(
         small = 20,
@@ -703,35 +853,28 @@ async def load_example(context, params):
         large = 500
     )
     n <- size_map[[size_param]]
-    
     set.seed(42)  # For reproducible examples
-    
     if (dataset_name == "sales") {{
         # Sales and marketing data
         months <- 1:n
         marketing_spend <- round(rnorm(n, 1000, 200), 0)
         sales <- round(50 + 4.5 * marketing_spend + rnorm(n, 0, 500), 0)
-        
         # Add some seasonal effect
         seasonal <- 200 * sin(2 * pi * months / 12)
         sales <- sales + seasonal
-        
         data <- data.frame(
             month = months,
             marketing_spend = pmax(marketing_spend, 0),
             sales = pmax(sales, 0),
             quarter = paste0("Q", ceiling(months %% 12 / 3))
         )
-        
         description <- "Sales and marketing spend data with seasonal patterns"
-        
     }} else if (dataset_name == "economics") {{
         # Economic indicators
         years <- seq(2000, 2000 + n/4, length.out = n)
         gdp_growth <- round(rnorm(n, 2.5, 1.2), 2)
         unemployment <- round(8 - 0.8 * gdp_growth + rnorm(n, 0, 0.5), 1)
         investment <- round(18 + 0.3 * gdp_growth + rnorm(n, 0, 2), 1)
-        
         data <- data.frame(
             year = years,
             gdp_growth = gdp_growth,
@@ -739,22 +882,17 @@ async def load_example(context, params):
             investment = pmax(investment, 10),
             country = sample(c("USA", "GBR", "DEU", "FRA"), n, replace = TRUE)
         )
-        
         description <- "Economic indicators demonstrating Okun's Law and investment relationships"
-        
     }} else if (dataset_name == "customers") {{
         # Customer data for churn analysis
         customer_id <- 1:n
         tenure_months <- sample(1:72, n, replace = TRUE)
         monthly_charges <- round(runif(n, 20, 120), 2)
         total_charges <- round(tenure_months * monthly_charges + rnorm(n, 0, 100), 2)
-        
         # Churn probability based on tenure and charges
         churn_prob <- plogis(-2 + -0.05 * tenure_months + 0.02 * monthly_charges)
         churned <- rbinom(n, 1, churn_prob)
-        
         age <- sample(18:80, n, replace = TRUE)
-        
         data <- data.frame(
             customer_id = customer_id,
             age = age,
@@ -764,39 +902,30 @@ async def load_example(context, params):
             churned = churned,
             contract_type = sample(c("Month-to-month", "One year", "Two year"), n, replace = TRUE, prob = c(0.5, 0.3, 0.2))
         )
-        
         description <- "Customer data for churn prediction analysis"
-        
     }} else if (dataset_name == "timeseries") {{
         # Time series data with trend and seasonality
         time_points <- 1:n
         trend <- 0.5 * time_points
         seasonal <- 10 * sin(2 * pi * time_points / 12) + 5 * cos(2 * pi * time_points / 4)
         noise <- rnorm(n, 0, 3)
-        
         value <- 100 + trend + seasonal + noise
-        
         data <- data.frame(
             time = time_points,
             value = round(value, 2),
             month = rep(1:12, length.out = n),
             year = rep(2020:(2020 + ceiling(n/12)), each = 12)[1:n]
         )
-        
         description <- "Time series data with trend and seasonal components"
-        
     }} else if (dataset_name == "survey") {{
         # Survey data with Likert scales
         respondent_id <- 1:n
         age <- sample(18:75, n, replace = TRUE)
         satisfaction <- sample(1:10, n, replace = TRUE, prob = c(0.05, 0.05, 0.1, 0.1, 0.15, 0.15, 0.15, 0.15, 0.05, 0.05))
-        
         # Purchase frequency correlated with satisfaction
         purchase_freq <- pmax(1, round(satisfaction * 0.8 + rnorm(n, 0, 1.5)), 0)
-        
         education <- sample(c("High School", "Bachelor", "Master", "PhD"), n, replace = TRUE, prob = c(0.3, 0.4, 0.25, 0.05))
         income_bracket <- sample(c("< $30k", "$30-50k", "$50-75k", "$75-100k", "> $100k"), n, replace = TRUE, prob = c(0.2, 0.25, 0.25, 0.2, 0.1))
-        
         data <- data.frame(
             respondent_id = respondent_id,
             age = age,
@@ -805,13 +934,10 @@ async def load_example(context, params):
             education = education,
             income_bracket = income_bracket
         )
-        
         description <- "Survey data with satisfaction and demographic variables"
-        
     }} else {{
         stop("Unknown dataset name")
     }}
-    
     # Add noise if requested
     if (add_noise) {{
         # Add missing values randomly (5-10% missing)
@@ -822,7 +948,6 @@ async def load_example(context, params):
                 data[missing_indices, col] <- NA
             }}
         }}
-        
         # Add some outliers to numeric columns
         numeric_cols <- names(data)[sapply(data, is.numeric)]
         for (col in numeric_cols) {{
@@ -835,7 +960,6 @@ async def load_example(context, params):
             }}
         }}
     }}
-    
     # Calculate basic statistics
     numeric_vars <- names(data)[sapply(data, is.numeric)]
     stats <- list()
@@ -850,9 +974,8 @@ async def load_example(context, params):
             )
         }}
     }}
-    
     result <- list(
-        data = data,
+        data = as.list(data),  # Convert to column-wise format for schema compatibility
         metadata = list(
             name = dataset_name,
             description = description,
@@ -865,11 +988,10 @@ async def load_example(context, params):
         suggested_analyses = list(),
         variable_info = list(
             numeric_variables = numeric_vars,
-            categorical_variables = names(data)[sapply(data, function(x) is.factor(x) || is.character(x))],
-            variable_types = sapply(data, class)
+            categorical_variables = as.list(names(data)[sapply(data, function(x) is.factor(x) || is.character(x))]),
+            variable_types = as.list(setNames(sapply(data, class), names(data)))
         )
     )
-    
     # Add suggested analyses based on dataset
     if (dataset_name == "sales") {{
         result$suggested_analyses <- c(
@@ -902,19 +1024,16 @@ async def load_example(context, params):
             "Multiple regression with demographic controls"
         )
     }}
-    
     result
     """
-
     try:
-        result = execute_r_script(r_script, params)
+        result = await execute_r_script_async(r_script, params)
         await context.info(
             "Example dataset loaded successfully",
             rows=result["metadata"]["rows"],
             columns=result["metadata"]["columns"],
         )
         return result
-
     except Exception as e:
         await context.error("Failed to load example dataset", error=str(e))
         return {
@@ -924,7 +1043,7 @@ async def load_example(context, params):
                 "name": dataset_name,
                 "rows": 0,
                 "columns": 0,
-                "description": "Failed to load"
+                "description": "Failed to load",
             },
-            "suggested_analyses": []
+            "suggested_analyses": [],
         }
